from .Relationship import Relationship
from .SchemaEntry import SchemaEntry
from .AttributeReference import AttributeReference
from .utils import String
from collections import OrderedDict


class SingleKeyRelationship(Relationship):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("$type", String, "SingleKeyRelationship"),
            SchemaEntry("fromAttribute", AttributeReference),
            SchemaEntry("toAttribute", AttributeReference)
        ]
        super().__init__(self.schema)

    def validate(self):
        super().validate()
        className = self.__class__.__name__
        if self.fromAttribute is None:
            raise ValueError("%s.fromAttribute is not set" % (className, ))
        if self.toAttribute is None:
            raise ValueError("%s.toAttribute is not set" % (className, ))
        if self.fromAttribute == self.toAttribute:
            raise ValueError("%s must exist between different attribute references" % (className, ))

    def toJson(self):
        result = OrderedDict()
        result["$type"] = "SingleKeyRelationship"
        entity = super().toJson()
        for k, v in entity.items():
            result[k] = v
        return result



from .Base import Base
from .SchemaEntry import SchemaEntry
from .Annotation import AnnotationCollection
from .ObjectCollection import ObjectCollection
from .utils import String
import re


class MetadataObject(Base):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("name", String),
            SchemaEntry("description", String),
            SchemaEntry("annotations", AnnotationCollection)
        ]
        super().__init__(self.schema)

    nameLengthMin = 1
    nameLengthMax = 256
    invalidNameRegex = re.compile("^\\s|\\s$")
    descriptionLengthMax = 4000

    def __repr__(self):
        name = getattr(self, "name", None)
        className = self.__class__.__name__
        if name:
            return "<%s '%s'>" % (className, name)
        else:
            return "<%s>" % (className, )
    
    def validate(self):
        if self.name is not None:
            if len(self.name) > self.nameLengthMax or len(self.name) < self.nameLengthMin:
                raise ValueError("Length of %s.name (%d) is not between %d and %d." % (className, len(self.name), self.nameLengthMin, self.nameLengthMax))
            if self.invalidNameRegex.search(self.name):
                raise ValueError("%s.name cannot contain leading or trailing blank spaces or consist only of whitespace." % (className, ))
        if self.description is not None and len(self.description) > self.descriptionLengthMax:
            raise ValueError("Length of %s.description (%d) may not exceed %d." % (className, len(self.name), self.nameLengthMin, self.nameLengthMax))


class MetadataObjectCollection(ObjectCollection):
    def __getitem__(self, index):
        if type(index) == str:
            index = next((i for i, item in enumerate(self) if item.name.lower() == index.lower()), None)
        if index is None:
            return None
        return super(MetadataObjectCollection, self).__getitem__(index)




from .MetadataObject import MetadataObject
from .MetadataObject import MetadataObjectCollection
from .SchemaEntry import SchemaEntry
from .CdmDataType import DataType


class Attribute(MetadataObject):
    def __init__(self, schema=[]):
        self.schema = schema + [
            SchemaEntry("dataType", DataType),
        ]
        super().__init__(self.schema)

    def __repr__(self):
        return "<[%s]>" % (getattr(self, "name", "(unnamed)"), )

class AttributeCollection(MetadataObjectCollection):
    itemType = Attribute


from .Reader import Reader
from azure.storage.blob import BlockBlobService
from io import StringIO
import json
import pandas as pd


class ADLSPandasReader(Reader):
    """
        Read dataframe/json to specified blob storage location
    """
    def __init__(self, account_name, account_key):
        self.account_name = account_name
        self.account_key = account_key
        self.block_blob_service = BlockBlobService(account_name=self.account_name, account_key=self.account_key)
    def read_df(self, locations, headers, dtypes):
        """
            Read csv files from the specified locations
        """
        dfs = pd.DataFrame()
        for blob_location in locations:
            location_list = blob_location.split('/',3)
            container_name = location_list[2].split('@')[0]
            csv_content = self.block_blob_service.get_blob_to_text(container_name, location_list[3] ).content
            df = pd.read_csv(StringIO(csv_content), names=headers,header=0 )
            dfs=dfs.append(df)
        return dfs